declare class API {
    constructor(kwargs?: Record<string, unknown>)
    get(url: string, options?: APIOptions): unknown
    post(url: string, options?: APIOptions): unknown
    put(url: string, options?: APIOptions): unknown
    delete(url: string, options?: APIOptions): unknown
    patch(url: string, options?: APIOptions): unknown
    options(url: string, options?: APIOptions): unknown
}

type APIOptions = {
    params?: Record<string, unknown>,
    headers?: HeadersInit,
    kwargs?: Record<string, unknown>,
    handle?: boolean,
    data?: BodyInit | JSON,
    dataJ?: JSON,
    dataM?: Record<string, unknown>
}

declare class YoniusError {
    constructor(message: string, code?: number)
    isClient(): boolean
    isServer(): boolean
}

declare class OperationalError extends YoniusError {
    constructor(message?: string, code?: number)
}

declare class NotFoundError extends OperationalError {}

declare class NotImplementedError extends OperationalError {}

declare class ValidationError extends OperationalError {}

declare function register(name: string, value: unknown): void;
declare function unregister(name: string): void;
declare function request(name: string): unknown;

declare class Model {
    static niw<T = Model>(this: { new(): T }): T
    static find<T = Model>(this: { new(): T }, params?: Record<string, unknown>): T[]
    static get<T = Model>(this: { new(): T }, params?: Record<string, unknown>): T

    constructor(options?: { fill?: boolean })
    apply<T = Model>(this: T, model: Record<string, unknown>): Promise<T>
    save<T = Model>(this: T): Promise<T>
    delete<T = Model>(this: T, options?: {
        preDelete?: boolean,
        postDelete?: boolean,
        beforeCallbacks?: ((self: T, model: Record<string, unknown>) => void)[],
        afterCallbacks?: ((self: T, model: Record<string, unknown>) => void)[]
    }): Promise<T>

    validate(): Promise<void>
    _validate(): IterableIterator<Error>
}

declare class ModelStore extends Model {
    static get schema(): Record<string, unknown>
    static get idName(): string
}

declare function verify(condition: boolean, message?: string, code?: number, exception?: unknown, kwargs?: unknown): void;

declare function load(names?: Array<string>, path?: string, encoding?: string, force?: boolean, ctx?: unknown): Promise<void>;
declare function conf(name: string, fallback?: unknown, cast?: string, ctx?: unknown): unknown;

declare function getObject(
    params: Record<string, unknown>,
    options: {
        alias?: boolean,
        page?: boolean,
        find?: boolean,
        norm?: boolean
    }
): Record<string, unknown>;

declare function initMongo(
    mongoose: unknown,
    uri: string
): Promise<void>;

declare function destroyMongo(
    mongoose: unknown
): void;

declare function eq<T>(
    valueC: T,
    message?: string
): (value?: T, ctx?: unknown) => boolean;

declare function gt<T>(
    valueC: T,
    message?: string
): (value?: T, ctx?: unknown) => boolean;

declare function gte<T>(
    valueC: T,
    message?: string
): (value?: T, ctx?: unknown) => boolean;

declare function notEmpty(
    message?: string
): (value?: string | unknown[], ctx?: unknown) => boolean;

declare function isIn<T>(
    valueC: T[],
    message?: string
): (value?: T, ctx?: unknown) => boolean;

declare function isUpper(
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function isLower(
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function isSimple(
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function isEmail(
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function isUrl(
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function isRegex(
    regex: string,
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function stringGt(
    valueC: number,
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function stringLt(
    valueC: number,
    message?: string
): (value?: string, ctx?: unknown) => boolean;

declare function stringEq(
    valueC: number,
    message?: string
): (value?: string, ctx?: unknown) => boolean;

export { API, Model, ModelStore, NotFoundError, NotImplementedError, OperationalError, ValidationError, YoniusError, conf, destroyMongo, eq, getObject, gt, gte, initMongo, isEmail, isIn, isLower, isRegex, isSimple, isUpper, isUrl, load, notEmpty, register, request, stringEq, stringGt, stringLt, unregister, verify };
